@@ -18,6 +18,7 @@ from barbican import api
 from barbican.api import controllers
 from barbican.api.controllers import acls
 from barbican.api.controllers import secretmeta
+from barbican.common import accept
 from barbican.common import exception
 from barbican.common import hrefs
 from barbican.common import quota
@@ -40,6 +41,11 @@ def _secret_not_found():
                          'another castle.'))
 
 
+def _invalid_secret_id():
+    """Throw exception indicating secret id is invalid."""
+    pecan.abort(404, u._('Not Found. Provided secret id is invalid.'))
+
+
 def _secret_payload_not_found():
     """Throw exception indicating secret's payload is not found."""
     pecan.abort(404, u._('Not Found. Sorry but your secret has no payload.'))
@@ -99,7 +105,7 @@ class SecretController(controllers.ACLMixin):
                     pecan.abort(405)
         else:
             # only 'acl' and 'metadata' as sub-resource is supported
-            pecan.abort(405)
+            pecan.abort(404)
 
     @pecan.expose(generic=True)
     def index(self, **kwargs):
@@ -113,12 +119,12 @@ class SecretController(controllers.ACLMixin):
         if controllers.is_json_request_accept(pecan.request):
             resp = self._on_get_secret_metadata(self.secret, **kwargs)
 
-            LOG.info(u._LI('Retrieved secret metadata for project: %s'),
+            LOG.info('Retrieved secret metadata for project: %s',
                      external_project_id)
             return resp
         else:
-            LOG.warning(u._LW('Decrypted secret %s requested using deprecated '
-                        'API call.'), self.secret.id)
+            LOG.warning('Decrypted secret %s requested using deprecated '
+                        'API call.', self.secret.id)
             return self._on_get_secret_payload(self.secret,
                                                external_project_id,
                                                **kwargs)
@@ -154,8 +160,11 @@ class SecretController(controllers.ACLMixin):
         project = res.get_or_create_project(external_project_id)
 
         # default to application/octet-stream if there is no Accept header
-        accept_header = getattr(pecan.request.accept, 'header_value',
-                                'application/octet-stream')
+        if (type(pecan.request.accept) is accept.NoHeaderType or
+                not pecan.request.accept.header_value):
+            accept_header = 'application/octet-stream'
+        else:
+            accept_header = pecan.request.accept.header_value
         pecan.override_template('', accept_header)
 
         # check if payload exists before proceeding
@@ -197,7 +206,7 @@ class SecretController(controllers.ACLMixin):
                                            external_project_id,
                                            **kwargs)
 
-        LOG.info(u._LI('Retrieved secret payload for project: %s'),
+        LOG.info('Retrieved secret payload for project: %s',
                  external_project_id)
         return resp
 
@@ -238,7 +247,7 @@ class SecretController(controllers.ACLMixin):
             secret_model=self.secret,
             project_model=project_model,
             transport_key_id=transport_key_id)
-        LOG.info(u._LI('Updated secret for project: %s'), external_project_id)
+        LOG.info('Updated secret for project: %s', external_project_id)
 
     @index.when(method='DELETE')
     @utils.allow_all_content_types
@@ -246,7 +255,7 @@ class SecretController(controllers.ACLMixin):
     @controllers.enforce_rbac('secret:delete')
     def on_delete(self, external_project_id, **kwargs):
         plugin.delete_secret(self.secret, external_project_id)
-        LOG.info(u._LI('Deleted secret for project: %s'), external_project_id)
+        LOG.info('Deleted secret for project: %s', external_project_id)
 
 
 class SecretsController(controllers.ACLMixin):
@@ -319,6 +328,8 @@ class SecretsController(controllers.ACLMixin):
         # check, the execution only gets here if authentication of the user was
         # previously successful.
 
+        if not utils.validate_id_is_uuid(secret_id):
+            _invalid_secret_id()()
         secret = self.secret_repo.get_secret_by_id(
             entity_id=secret_id, suppress_exception=True)
         if not secret:
@@ -337,8 +348,8 @@ class SecretsController(controllers.ACLMixin):
         def secret_fields(field):
             return putil.mime_types.augment_fields_with_content_types(field)
 
-        LOG.debug(u._('Start secrets on_get '
-                      'for project-ID %s:'), external_project_id)
+        LOG.debug('Start secrets on_get '
+                  'for project-ID %s:', external_project_id)
 
         name = kw.get('name', '')
         if name:
@@ -372,6 +383,7 @@ class SecretsController(controllers.ACLMixin):
             alg=kw.get('alg'),
             mode=kw.get('mode'),
             bits=bits,
+            secret_type=kw.get('secret_type'),
             suppress_exception=True,
             acl_only=kw.get('acl_only'),
             user_id=user_id,
@@ -397,7 +409,7 @@ class SecretsController(controllers.ACLMixin):
             )
             secrets_resp_overall.update({'total': total})
 
-        LOG.info(u._LI('Retrieved secret list for project: %s'),
+        LOG.info('Retrieved secret list for project: %s',
                  external_project_id)
         return secrets_resp_overall
 
@@ -406,7 +418,7 @@ class SecretsController(controllers.ACLMixin):
     @controllers.enforce_rbac('secrets:post')
     @controllers.enforce_content_types(['application/json'])
     def on_post(self, external_project_id, **kwargs):
-        LOG.debug(u._('Start on_post for project-ID %s:...'),
+        LOG.debug('Start on_post for project-ID %s:...',
                   external_project_id)
 
         data = api.load_body(pecan.request, validator=self.validator)
@@ -433,12 +445,12 @@ class SecretsController(controllers.ACLMixin):
             transport_key_id=data.get('transport_key_id'))
 
         url = hrefs.convert_secret_to_href(new_secret.id)
-        LOG.debug(u._('URI to secret is %s'), url)
+        LOG.debug('URI to secret is %s', url)
 
         pecan.response.status = 201
         pecan.response.headers['Location'] = url
 
-        LOG.info(u._LI('Created a secret for project: %s'),
+        LOG.info('Created a secret for project: %s',
                  external_project_id)
         if transport_key_model is not None:
             tkey_url = hrefs.convert_transport_key_to_href(
