@@ -15,13 +15,13 @@
 
 import base64
 import copy
+from cryptography.hazmat.backends import default_backend
+from cryptography.hazmat.primitives import serialization
 import datetime
 import os
-import uuid
+from oslo_utils import uuidutils
+import time
 
-from Crypto.PublicKey import RSA
-from Crypto.Util import asn1
-from oslo_config import cfg
 import pki
 
 subcas_available = True
@@ -39,51 +39,21 @@ import pki.kra
 import pki.profile
 from requests import exceptions as request_exceptions
 
-from barbican.common import config
 from barbican.common import exception
 from barbican.common import utils
 from barbican import i18n as u
+
+# we want to keep the dogtag config options separated. That way we
+# do not need to import every dogtag requirement to generate the
+# sample config
+import barbican.plugin.dogtag_config_opts  # noqa
 import barbican.plugin.interface.certificate_manager as cm
 import barbican.plugin.interface.secret_store as sstore
 
-CONF = config.new_config()
+# reuse the conf object to not call config.new_config() twice
+CONF = barbican.plugin.dogtag_config_opts.CONF
 LOG = utils.getLogger(__name__)
 
-dogtag_plugin_group = cfg.OptGroup(name='dogtag_plugin',
-                                   title="Dogtag Plugin Options")
-dogtag_plugin_opts = [
-    cfg.StrOpt('pem_path',
-               help=u._('Path to PEM file for authentication')),
-    cfg.StrOpt('dogtag_host',
-               default="localhost",
-               help=u._('Hostname for the Dogtag instance')),
-    cfg.StrOpt('dogtag_port',
-               default="8443",
-               help=u._('Port for the Dogtag instance')),
-    cfg.StrOpt('nss_db_path',
-               help=u._('Path to the NSS certificate database')),
-    cfg.StrOpt('nss_password',
-               help=u._('Password for the NSS certificate databases'),
-               secret=True),
-    cfg.StrOpt('simple_cmc_profile',
-               help=u._('Profile for simple CMC requests')),
-    cfg.StrOpt('auto_approved_profiles',
-               default="caServerCert",
-               help=u._('List of automatically approved enrollment profiles')),
-    cfg.StrOpt('ca_expiration_time',
-               default=cm.CA_INFO_DEFAULT_EXPIRATION_DAYS,
-               help=u._('Time in days for CA entries to expire')),
-    cfg.StrOpt('plugin_working_dir',
-               help=u._('Working directory for Dogtag plugin')),
-    cfg.StrOpt('plugin_name',
-               help=u._('User friendly plugin name'),
-               default='Dogtag KRA'),
-]
-
-CONF.register_group(dogtag_plugin_group)
-CONF.register_opts(dogtag_plugin_opts, group=dogtag_plugin_group)
-config.parse_args(CONF)
-
 CERT_HEADER = "-----BEGIN CERTIFICATE-----"
 CERT_FOOTER = "-----END CERTIFICATE-----"
 
@@ -100,8 +70,8 @@ def _create_nss_db_if_needed(nss_db_path, nss_password):
             nss_db_path, nss_password, over_write=True)
         return True
     else:
-        LOG.info(u._LI("The nss_db_path provided already exists, so the "
-                       "database is assumed to be already set up."))
+        LOG.info("The nss_db_path provided already exists, so the "
+                 "database is assumed to be already set up.")
         return False
 
 
@@ -118,8 +88,8 @@ def _setup_nss_db_services(conf):
     nss_db_path, nss_password = (conf.dogtag_plugin.nss_db_path,
                                  conf.dogtag_plugin.nss_password)
     if nss_db_path is None:
-        LOG.warning(u._LW("nss_db_path was not provided so the crypto "
-                          "provider functions were not initialized."))
+        LOG.warning("nss_db_path was not provided so the crypto "
+                    "provider functions were not initialized.")
         return None
     if nss_password is None:
         raise ValueError(u._("nss_password is required"))
@@ -142,17 +112,18 @@ def _import_kra_transport_cert_to_nss_db(conf, crypto):
         crypto.import_cert(KRA_TRANSPORT_NICK, transport_cert, "u,u,u")
     except Exception as e:
         LOG.error("Error in importing transport cert."
-                  " KRA may not be enabled: " + str(e))
+                  " KRA may not be enabled: %s", e)
 
 
 def create_connection(conf, subsystem_path):
     pem_path = conf.dogtag_plugin.pem_path
     if pem_path is None:
         raise ValueError(u._("pem_path is required"))
+    # port is string type in PKIConnection
     connection = pki.client.PKIConnection(
         'https',
         conf.dogtag_plugin.dogtag_host,
-        conf.dogtag_plugin.dogtag_port,
+        str(conf.dogtag_plugin.dogtag_port),
         subsystem_path)
     connection.set_authentication_cert(pem_path)
     return connection
@@ -178,6 +149,14 @@ class DogtagPluginNotSupportedException(exception.NotSupported):
         super(DogtagPluginNotSupportedException, self).__init__(message)
 
 
+class DogtagPluginArchivalException(exception.BarbicanException):
+    message = u._("Key archival failed.  Error returned from KRA.")
+
+
+class DogtagPluginGenerationException(exception.BarbicanException):
+    message = u._("Key generation failed.  Error returned from KRA.")
+
+
 class DogtagKRAPlugin(sstore.SecretStoreBase):
     """Implementation of the secret store plugin with KRA as the backend."""
 
@@ -198,7 +177,7 @@ class DogtagKRAPlugin(sstore.SecretStoreBase):
 
     def __init__(self, conf=CONF):
         """Constructor - create the keyclient."""
-        LOG.debug(u._("starting DogtagKRAPlugin init"))
+        LOG.debug("starting DogtagKRAPlugin init")
         connection = create_connection(conf, 'kra')
 
         # create kraclient
@@ -207,8 +186,9 @@ class DogtagKRAPlugin(sstore.SecretStoreBase):
 
         self.keyclient.set_transport_cert(KRA_TRANSPORT_NICK)
         self.plugin_name = conf.dogtag_plugin.plugin_name
+        self.retries = conf.dogtag_plugin.retries
 
-        LOG.debug(u._("completed DogtagKRAPlugin init"))
+        LOG.debug("completed DogtagKRAPlugin init")
 
     def get_plugin_name(self):
         return self.plugin_name
@@ -233,25 +213,41 @@ class DogtagKRAPlugin(sstore.SecretStoreBase):
         the key_id
         """
         data_type = key.KeyClient.PASS_PHRASE_TYPE
-        client_key_id = uuid.uuid4().hex
-        if secret_dto.transport_key is not None:
-            # TODO(alee-3) send the transport key with the archival request
-            # once the Dogtag Client API changes.
-            response = self.keyclient.archive_pki_options(
-                client_key_id,
-                data_type,
-                secret_dto.secret,
-                key_algorithm=None,
-                key_size=None)
-        else:
-            response = self.keyclient.archive_key(
-                client_key_id,
-                data_type,
-                secret_dto.secret,
-                key_algorithm=None,
-                key_size=None)
+        key_id = None
 
-        meta_dict = {DogtagKRAPlugin.KEY_ID: response.get_key_id()}
+        attempts = 0
+        offset_time = 1
+        while attempts <= self.retries and key_id is None:
+            client_key_id = uuidutils.generate_uuid(dashed=False)
+            if secret_dto.transport_key is not None:
+                # TODO(alee-3) send the transport key with the archival request
+                # once the Dogtag Client API changes.
+                response = self.keyclient.archive_pki_options(
+                    client_key_id,
+                    data_type,
+                    secret_dto.secret,
+                    key_algorithm=None,
+                    key_size=None)
+            else:
+                response = self.keyclient.archive_key(
+                    client_key_id,
+                    data_type,
+                    secret_dto.secret,
+                    key_algorithm=None,
+                    key_size=None)
+
+            key_id = response.get_key_id()
+
+            if key_id is None:
+                LOG.warning("key_id is None.  attempts: {}".format(attempts))
+                attempts += 1
+                time.sleep(offset_time)
+                offset_time += 1
+
+        if key_id is None:
+            raise DogtagPluginArchivalException
+
+        meta_dict = {DogtagKRAPlugin.KEY_ID: key_id}
 
         self._store_secret_attributes(meta_dict, secret_dto)
         return meta_dict
@@ -320,51 +316,32 @@ class DogtagKRAPlugin(sstore.SecretStoreBase):
                 # as it is treated as an attribute of the asymmetric key pair
                 # stored in the KRA database.
 
-                if key_spec.alg is None:
-                    raise sstore.SecretAlgorithmNotSupportedException('None')
-
                 key_info = self.keyclient.get_key_info(key_id)
-                if key_spec.alg.upper() == key.KeyClient.RSA_ALGORITHM:
-                    recovered_key = (RSA.importKey(key_info.public_key)
-                                     .publickey()
-                                     .exportKey('PEM')).encode('utf-8')
-                elif key_spec.alg.upper() == key.KeyClient.DSA_ALGORITHM:
-                    pub_seq = asn1.DerSequence()
-                    pub_seq[:] = key_info.public_key
-                    recovered_key = (
-                        ("%s\n%s%s" %
-                         (DogtagKRAPlugin.DSA_PUBLIC_KEY_HEADER,
-                          pub_seq.encode().encode("base64"),
-                          DogtagKRAPlugin.DSA_PUBLIC_KEY_FOOTER)
-                         ).encode('utf-8')
-                    )
-                else:
-                    raise sstore.SecretAlgorithmNotSupportedException(
-                        key_spec.alg.upper()
-                    )
+                recovered_key = serialization.load_der_public_key(
+                    key_info.public_key,
+                    backend=default_backend()
+                ).public_bytes(
+                    serialization.Encoding.PEM,
+                    serialization.PublicFormat.PKCS1)
 
             elif secret_type == sstore.SecretType.PRIVATE:
                 key_data = self.keyclient.retrieve_key(key_id)
-                if key_spec.alg.upper() == key.KeyClient.RSA_ALGORITHM:
-                    recovered_key = (
-                        (RSA.importKey(key_data.data)
-                         .exportKey('PEM', passphrase, 8))
-                        .encode('utf-8')
-                    )
-                elif key_spec.alg.upper() == key.KeyClient.DSA_ALGORITHM:
-                    pub_seq = asn1.DerSequence()
-                    pub_seq[:] = key_data.data
-                    recovered_key = (
-                        ("%s\n%s%s" %
-                         (DogtagKRAPlugin.DSA_PRIVATE_KEY_HEADER,
-                          pub_seq.encode().encode("base64"),
-                          DogtagKRAPlugin.DSA_PRIVATE_KEY_FOOTER)
-                         ).encode('utf-8')
-                    )
+                private_key = serialization.load_der_private_key(
+                    key_data.data,
+                    password=None,
+                    backend=default_backend()
+                )
+
+                if passphrase is not None:
+                    e_alg = serialization.BestAvailableEncryption(passphrase)
                 else:
-                    raise sstore.SecretAlgorithmNotSupportedException(
-                        key_spec.alg.upper()
-                    )
+                    e_alg = serialization.NoEncryption()
+
+                recovered_key = private_key.private_bytes(
+                    encoding=serialization.Encoding.PEM,
+                    format=serialization.PrivateFormat.PKCS8,
+                    encryption_algorithm=e_alg
+                )
         else:
             # TODO(alee-3) send transport key as well when dogtag client API
             # changes in case the transport key has changed.
@@ -411,7 +388,6 @@ class DogtagKRAPlugin(sstore.SecretStoreBase):
         usages = [key.SymKeyGenerationRequest.DECRYPT_USAGE,
                   key.SymKeyGenerationRequest.ENCRYPT_USAGE]
 
-        client_key_id = uuid.uuid4().hex
         algorithm = self._map_algorithm(key_spec.alg.lower())
 
         if algorithm is None:
@@ -422,11 +398,27 @@ class DogtagKRAPlugin(sstore.SecretStoreBase):
                 u._("Passphrase encryption is not supported for symmetric"
                     " key generating algorithms."))
 
-        response = self.keyclient.generate_symmetric_key(
-            client_key_id,
-            algorithm,
-            key_spec.bit_length,
-            usages)
+        key_id = None
+        attempts = 0
+        offset_time = 1
+        while attempts <= self.retries and key_id is None:
+            client_key_id = uuidutils.generate_uuid()
+            response = self.keyclient.generate_symmetric_key(
+                client_key_id,
+                algorithm,
+                key_spec.bit_length,
+                usages)
+            key_id = response.get_key_id()
+
+            if key_id is None:
+                LOG.warning("generate_symkey: key_id is None.  attempts: {}"
+                            .format(attempts))
+                attempts += 1
+                time.sleep(offset_time)
+                offset_time += 1
+
+        if key_id is None:
+            raise DogtagPluginGenerationException
 
         # Barbican expects stored keys to be base 64 encoded.  We need to
         # add flag to the keyclient.generate_symmetric_key() call above
@@ -438,7 +430,6 @@ class DogtagKRAPlugin(sstore.SecretStoreBase):
         # key encoded data.
         return {DogtagKRAPlugin.ALG: key_spec.alg,
                 DogtagKRAPlugin.BIT_LENGTH: key_spec.bit_length,
-                DogtagKRAPlugin.SECRET_MODE: key_spec.mode,
                 DogtagKRAPlugin.KEY_ID: response.get_key_id(),
                 DogtagKRAPlugin.GENERATED: True}
 
@@ -451,7 +442,7 @@ class DogtagKRAPlugin(sstore.SecretStoreBase):
         usages = [key.AsymKeyGenerationRequest.DECRYPT_USAGE,
                   key.AsymKeyGenerationRequest.ENCRYPT_USAGE]
 
-        client_key_id = uuid.uuid4().hex
+        client_key_id = uuidutils.generate_uuid()
         algorithm = self._map_algorithm(key_spec.alg.lower())
         passphrase = key_spec.passphrase
 
@@ -468,7 +459,7 @@ class DogtagKRAPlugin(sstore.SecretStoreBase):
                 )
 
             stored_passphrase_info = self.keyclient.archive_key(
-                uuid.uuid4().hex,
+                uuidutils.generate_uuid(),
                 self.keyclient.PASS_PHRASE_TYPE,
                 base64.b64encode(passphrase))
 
@@ -671,7 +662,7 @@ def _catch_subca_deletion_exceptions(ca_related_function):
         try:
             return ca_related_function(self, *args, **kwargs)
         except pki.ResourceNotFoundException as e:
-            LOG.warning(u._LI("Sub-CA already deleted"))
+            LOG.warning("Sub-CA already deleted")
             pass
         except pki.PKIException as e:
             raise exception.SubCADeletionErrors(reason=e.message)
@@ -730,7 +721,7 @@ class DogtagCAPlugin(cm.CertificatePluginBase):
                         "%Y-%m-%d %H:%M:%S.%f"
                     )
             except (ValueError, TypeError):
-                LOG.warning(u._LI("Invalid data read from expiration file"))
+                LOG.warning("Invalid data read from expiration file")
                 self.expiration = datetime.utcnow()
         return self._expiration
 
@@ -769,15 +760,15 @@ class DogtagCAPlugin(cm.CertificatePluginBase):
                 feature_client = feature.FeatureClient(connection)
                 authority_feature = feature_client.get_feature("authority")
                 if authority_feature.enabled:
-                    LOG.info(u._LI("Sub-CAs are enabled by Dogtag server"))
+                    LOG.info("Sub-CAs are enabled by Dogtag server")
                     return True
                 else:
-                    LOG.info(u._LI("Sub-CAs are not enabled by Dogtag server"))
+                    LOG.info("Sub-CAs are not enabled by Dogtag server")
             except (request_exceptions.HTTPError,
                     pki.ResourceNotFoundException):
-                LOG.info(u._LI("Sub-CAs are not supported by Dogtag server"))
+                LOG.info("Sub-CAs are not supported by Dogtag server")
         else:
-            LOG.info(u._LI("Sub-CAs are not supported by Dogtag client"))
+            LOG.info("Sub-CAs are not supported by Dogtag client")
         return False
 
     def _get_request_id(self, order_id, plugin_meta, operation):
