@@ -62,7 +62,7 @@ def get_public_key_req():
 
 def get_certificate_req():
     return {'name': 'mycertificate',
-            'payload_content_type': 'application/pkix-cert',
+            'payload_content_type': 'application/octet-stream',
             'payload_content_encoding': 'base64',
             'algorithm': 'rsa',
             'bit_length': 2048,
@@ -212,7 +212,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
         result = self.validator.validate(self.secret_req)
 
         self.assertIn('expiration', result)
-        self.assertTrue(not result['expiration'])
+        self.assertFalse(result['expiration'])
 
     def test_should_raise_numeric_name(self):
         self.secret_req['name'] = 123
@@ -242,7 +242,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('bit_length', exception.invalid_property)
-        self.assertIn('bit_length', exception.message)
+        self.assertIn('bit_length', six.text_type(exception))
 
     def test_should_raise_non_integer_bit_length(self):
         self.secret_req['bit_length'] = "23"
@@ -253,7 +253,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('bit_length', exception.invalid_property)
-        self.assertIn('bit_length', exception.message)
+        self.assertIn('bit_length', six.text_type(exception))
 
     def test_should_raise_bit_length_less_than_min(self):
         self.secret_req['bit_length'] = 0
@@ -264,7 +264,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('bit_length', exception.invalid_property)
-        self.assertIn('bit_length', exception.message)
+        self.assertIn('bit_length', six.text_type(exception))
 
     def test_should_raise_bit_length_greater_than_max(self):
         self.secret_req['bit_length'] = 32768
@@ -275,7 +275,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('bit_length', exception.invalid_property)
-        self.assertIn('bit_length', exception.message)
+        self.assertIn('bit_length', six.text_type(exception))
 
     def test_should_raise_mode_length_greater_than_max(self):
         self.secret_req['mode'] = 'a' * 256
@@ -286,7 +286,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('mode', exception.invalid_property)
-        self.assertIn('mode', exception.message)
+        self.assertIn('mode',  six.text_type(exception))
 
     def test_should_raise_mode_is_non_string(self):
         self.secret_req['mode'] = 123
@@ -297,7 +297,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('mode', exception.invalid_property)
-        self.assertIn('mode', exception.message)
+        self.assertIn('mode', six.text_type(exception))
 
     def test_validation_should_raise_with_empty_payload(self):
         self.secret_req['payload'] = '   '
@@ -308,7 +308,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('payload', exception.invalid_property)
-        self.assertIn('payload', exception.message)
+        self.assertIn('payload', six.text_type(exception))
 
     def test_should_raise_already_expired(self):
         self.secret_req['expiration'] = '2004-02-28T19:14:44.180394'
@@ -319,7 +319,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('expiration', exception.invalid_property)
-        self.assertIn('expiration', exception.message)
+        self.assertIn('expiration', six.text_type(exception))
 
     def test_should_raise_expiration_nonsense(self):
         self.secret_req['expiration'] = 'nonsense'
@@ -330,7 +330,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('expiration', exception.invalid_property)
-        self.assertIn('expiration', exception.message)
+        self.assertIn('expiration', six.text_type(exception))
 
     def test_should_raise_expiration_is_non_string(self):
         self.secret_req['expiration'] = 123
@@ -341,7 +341,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('expiration', exception.invalid_property)
-        self.assertIn('expiration', exception.message)
+        self.assertIn('expiration', six.text_type(exception))
 
     def test_should_raise_expiration_greater_than_max(self):
         self.secret_req['expiration'] = 'a' * 256
@@ -352,7 +352,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('expiration', exception.invalid_property)
-        self.assertIn('expiration', exception.message)
+        self.assertIn('expiration', six.text_type(exception))
 
     def test_should_raise_algorithm_is_non_string(self):
         self.secret_req['algorithm'] = 123
@@ -363,7 +363,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('algorithm', exception.invalid_property)
-        self.assertIn('algorithm', exception.message)
+        self.assertIn('algorithm', six.text_type(exception))
 
     def test_should_raise_algorithm_greater_than_max(self):
         self.secret_req['algorithm'] = 'a' * 256
@@ -374,7 +374,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('algorithm', exception.invalid_property)
-        self.assertIn('algorithm', exception.message)
+        self.assertIn('algorithm', six.text_type(exception))
 
     def test_should_raise_all_nulls(self):
         self.secret_req = {'name': None,
@@ -415,9 +415,8 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
         try:
             self.validator.validate(self.secret_req)
         except excep.InvalidObject as e:
-            self.assertNotEqual(str(e), 'None')
-            self.assertIsNotNone(e.message)
-            self.assertNotEqual(e.message, 'None')
+            self.assertIsNotNone(e)
+            self.assertIsNotNone(six.text_type(e))
         else:
             self.fail('No validation exception was raised')
 
@@ -455,7 +454,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
             self.secret_req,
         )
         self.assertEqual('payload_content_type', exception.invalid_property)
-        self.assertIn('payload_content_type', exception.message)
+        self.assertIn('payload_content_type', six.text_type(exception))
 
     def test_should_raise_with_payload_content_encoding_greater_than_max(self):
         self.secret_req['payload_content_encoding'] = 'a' * 256
@@ -466,7 +465,7 @@ class WhenTestingSecretValidator(utils.BaseTestCase):
         )
         self.assertEqual('payload_content_encoding',
                          exception.invalid_property)
-        self.assertIn('payload_content_encoding', exception.message)
+        self.assertIn('payload_content_encoding', six.text_type(exception))
 
     def test_should_raise_with_plain_text_and_encoding(self):
         self.secret_req['payload_content_encoding'] = 'base64'
@@ -584,8 +583,7 @@ class WhenTestingContainerValidator(utils.BaseTestCase):
     def setUp(self):
         super(WhenTestingContainerValidator, self).setUp()
 
-        validators.CONF.set_override("host_href", "http://localhost:9311",
-                                     enforce_type=True)
+        validators.CONF.set_override("host_href", "http://localhost:9311")
 
         self.name = 'name'
         self.type = 'generic'
@@ -626,7 +624,7 @@ class WhenTestingContainerValidator(utils.BaseTestCase):
             self.container_req,
         )
         self.assertEqual('name', exception.invalid_property)
-        self.assertIn('name', exception.message)
+        self.assertIn('name', six.text_type(exception))
 
     def test_should_raise_nonstring_secret_name(self):
         self.secret_refs[0]["name"] = 5
@@ -664,14 +662,13 @@ class WhenTestingContainerValidator(utils.BaseTestCase):
     def test_should_raise_no_type(self):
         del self.container_req['type']
 
-        self.assertRaises(
+        exception = self.assertRaises(
             excep.InvalidObject,
             self.validator.validate,
             self.container_req,
         )
 
-        # TODO(hgedikli): figure out why invalid_property is null here
-        # self.assertEqual('type', e.exception.invalid_property)
+        self.assertIn('type', six.text_type(exception))
 
     def test_should_raise_empty_type(self):
         self.container_req['type'] = ''
@@ -859,8 +856,7 @@ class WhenTestingRSAContainerValidator(utils.BaseTestCase):
     def setUp(self):
         super(WhenTestingRSAContainerValidator, self).setUp()
 
-        validators.CONF.set_override("host_href", "http://localhost:9311",
-                                     enforce_type=True)
+        validators.CONF.set_override("host_href", "http://localhost:9311")
 
         self.name = 'name'
         self.type = 'rsa'
@@ -975,8 +971,7 @@ class WhenTestingCertificateContainerValidator(utils.BaseTestCase):
     def setUp(self):
         super(WhenTestingCertificateContainerValidator, self).setUp()
 
-        validators.CONF.set_override("host_href", "http://localhost:9311",
-                                     enforce_type=True)
+        validators.CONF.set_override("host_href", "http://localhost:9311")
 
         self.name = 'name'
         self.type = 'certificate'
@@ -1216,7 +1211,7 @@ class WhenTestingKeyTypeOrderValidator(utils.BaseTestCase):
     def test_should_pass_good_exp_meta_in_order_refs(self):
         self.key_order_req['meta']['algorithm'] = 'AES'
         ony_year_factor = datetime.timedelta(days=1 * 365)
-        date_after_year = datetime.datetime.now() + ony_year_factor
+        date_after_year = datetime.datetime.utcnow() + ony_year_factor
         date_after_year_str = date_after_year.strftime('%Y-%m-%d %H:%M:%S')
         self.key_order_req['meta']['expiration'] = date_after_year_str
         result = self.validator.validate(self.key_order_req)
