@@ -53,7 +53,8 @@ jIeFW9U1C6OcOGvm40Lr3UKzMa5Yrtq6MW4ri7uSCwKBgQDfdqVjT4uXmGwOh1z4
 Pzv6GCoc+6GobXg4DvvCUjP9MR+2+5sX0AY/f+aVCD05/Nj0RqpAwUc03zZU5ZtL
 2uNe6XDjEugfFtlzea6+rbD6KpFS+nxPJA8YyWYRpNhpRWGWQakHedr3BtMtGs0h
 pKNAQG72HKWtSfJQMXvn2RlicA==
------END PRIVATE KEY-----"""
+-----END PRIVATE KEY-----
+"""
 
 
 def get_private_key_der():
@@ -174,7 +175,8 @@ udQQn4RlVt+cOdjmP9t8eTHjuMr8eZsj3HJ8TFUONirg68yqowZUo5gZ3KRmMdCY
 Ak/rMhZh7JfKzpKgjzxS6NuGEJ/uP6a9QGMGmQGzE5fc6C7REI+rMUnLh3EvXvJ4
 qbQ8ZbGy0IKhlWhnRNde7LQveUB+1LyE27mM3Y2cARXNUoM/Bmf9oS0rB7oyYiEH
 LwIDAQAB
------END PUBLIC KEY-----"""
+-----END PUBLIC KEY-----
+"""
 
 
 def get_public_key_der():
@@ -226,7 +228,7 @@ def get_encrypted_private_key_pem():
     of the private_encrypted.pk8 file.
     """
 
-    return """-----BEGIN ENCRYPTED PRIVATE KEY-----
+    return b"""-----BEGIN ENCRYPTED PRIVATE KEY-----
 MIIE6TAbBgkqhkiG9w0BBQMwDgQIssadeQrYhhACAggABIIEyDNw3SV2b19yy4Q/
 kTbtJ/p2X2zKDqr7GgLeAowqqhcMfvprI7G8C0XtwxkR4SjMZUXNcmOwQB2kNKtK
 ZilCz6pSx81iUj4s1fU460XkhkIeV+F7aB2PsTG1oDfPCuzKFjT6EuSE6lFUH89r
@@ -268,7 +270,7 @@ def get_passphrase_txt():
     of the passphrase.txt file.
     """
 
-    return """password"""
+    return b"""password"""
 
 
 def get_csr_pem():
@@ -283,7 +285,7 @@ def get_csr_pem():
     of the csr.pem file.
     """
 
-    return """-----BEGIN CERTIFICATE REQUEST-----
+    return b"""-----BEGIN CERTIFICATE REQUEST-----
 MIICWzCCAUMCAQAwFjEUMBIGA1UEAwwLZXhhbXBsZS5jb20wggEiMA0GCSqGSIb3
 DQEBAQUAA4IBDwAwggEKAoIBAQCza2VoDXmBUMmwjFu9F6MM5q/AZ1WjnWA2YNdN
 y237TrGN/nobDDv8FBBpUPmHNZ04H1LyxFcP8ReFrcIXpifsReu2lAWaqRPxovu5
