@@ -19,10 +19,10 @@ import os
 from os import path
 import time
 import types
-import uuid
 
 import mock
 from oslo_config import cfg
+from oslo_utils import uuidutils
 import oslotest.base as oslotest
 from oslotest import createfile
 
@@ -49,6 +49,7 @@ def mock_pecan_request(test_instance, host=None):
     mock_req = patcher_obj.start()
     test_instance.addCleanup(patcher_obj.stop)
     mock_req.url = host
+    mock_req.environ = os.environ.copy()
 
 
 @contextmanager
@@ -66,7 +67,7 @@ class BarbicanAPIBaseTestCase(oslotest.BaseTestCase):
         context = barbican.context.RequestContext(
             roles=roles,
             user=user,
-            project=project_id,
+            project_id=project_id,
             is_admin=is_admin
         )
         context.policy_enforcer = policy_enforcer
@@ -78,7 +79,7 @@ class BarbicanAPIBaseTestCase(oslotest.BaseTestCase):
         database_utils.setup_in_memory_db()
 
         # Generic project id to perform actions under
-        self.project_id = str(uuid.uuid4())
+        self.project_id = generate_test_valid_uuid()
 
         # Build the test app
         wsgi_app = app.build_wsgi_app(
@@ -536,9 +537,6 @@ def setup_multiple_secret_store_plugins_conf(testcase, store_plugin_names,
 
 class MultipleBackendsTestCase(database_utils.RepositoryTestCase):
 
-    def setUp(self):
-        super(MultipleBackendsTestCase, self).setUp()
-
     def _mock_plugin_settings(self):
 
         kmip_conf = kss.CONF
@@ -584,7 +582,7 @@ def create_timestamp_w_tz_and_offset(timezone=None, days=0, hours=0, minutes=0,
     :param hours: The offset in hours
     :param minutes: The offset in minutes
 
-    :return a timestamp
+    :return: a timestamp
     """
     if timezone is None:
         timezone = time.strftime("%z")
@@ -612,20 +610,6 @@ def get_tomorrow_timestamp():
     return tomorrow.isoformat()
 
 
-def string_to_datetime(datetimestring, date_formats=None):
-    date_formats = date_formats or [
-        '%Y-%m-%d %H:%M:%S', '%Y-%m-%d %H:%M:%S.%f', '%Y-%m-%dT%H:%M:%S.%fZ',
-        '%Y-%m-%dT%H:%M:%S.%f', "%Y-%m-%dT%H:%M:%SZ", "%Y-%m-%dT%H:%M:%S"]
-
-    for dateformat in date_formats:
-        try:
-            return datetime.datetime.strptime(datetimestring, dateformat)
-        except ValueError:
-            continue
-    else:
-        raise
-
-
 def get_id_from_ref(ref):
     """Returns id from reference."""
     ref_id = None
@@ -640,6 +624,11 @@ def generate_test_uuid(tail_value=0):
                                                             pad=12)
 
 
+def generate_test_valid_uuid():
+    """Returns a valid uuid value, similar to uuid generated in barbican"""
+    return uuidutils.generate_uuid()
+
+
 def get_symmetric_key():
     s = b"MIICdgIBADANBgkqhkiG9w=="
     return s
@@ -673,5 +662,13 @@ def is_kmip_enabled():
     return os.environ.get('KMIP_PLUGIN_ENABLED') is not None
 
 
+def is_vault_enabled():
+    return os.environ.get('VAULT_PLUGIN_ENABLED') is not None
+
+
+def is_pkcs11_enabled():
+    return os.environ.get('PKCS11_PLUGIN_ENABLED') is not None
+
+
 class DummyClassForTesting(object):
     pass
