@@ -17,7 +17,7 @@ import abc
 import fixtures
 import logging
 import os
-import uuid
+from oslo_utils import uuidutils
 
 import oslotest.base as oslotest
 import six
@@ -37,8 +37,8 @@ conf_multiple_backends_enabled = CONF.keymanager.\
 
 class TestCase(oslotest.BaseTestCase):
     max_payload_size = 10000
-    max_sized_payload = 'a' * max_payload_size
-    oversized_payload = 'a' * (max_payload_size + 1)
+    max_sized_payload = b'a' * max_payload_size
+    oversized_payload = b'a' * (max_payload_size + 1)
     max_field_size = 255
     max_sized_field = 'a' * max_field_size
     oversized_field = 'a' * (max_field_size + 1)
@@ -99,7 +99,7 @@ class PagingTestCase(TestCase):
         super(PagingTestCase, self).tearDown()
 
     def _set_filter_field(self, model):
-        filter = str(uuid.uuid4())
+        filter = uuidutils.generate_uuid()
         self.set_filter_field(filter, model)
         return filter
 
@@ -128,7 +128,7 @@ class PagingTestCase(TestCase):
         detect duplicates across multiple calls)
         """
         self.assertIsNotNone(resources)
-        self.assertEqual(len(resources), expected_size)
+        self.assertEqual(expected_size, len(resources))
         self.assertEqual(next_ref_should_be_none, next_ref is None)
         self.assertEqual(prev_ref_should_be_none, prev_ref is None)
 
@@ -139,7 +139,7 @@ class PagingTestCase(TestCase):
         if len(self._all_fetched_resources):
             duplicates = [entity for entity in resources if entity in
                           self._all_fetched_resources]
-            self.assertEqual(len(duplicates), 0)
+            self.assertEqual(0, len(duplicates))
 
         # add to our running list of resource refs
         self._all_fetched_resources.extend(resources)
